import os
import pathlib

from yaml_pyconf.base import BaseConfig


class FlaskConfig(BaseConfig):
    _db_uri = None

    def __new__(
            cls,
            yaml_path=pathlib.Path(__file__).parent.joinpath("samples").joinpath("sample-yaml").joinpath("flask-choose-env.yaml"),
            dotenv_path=pathlib.Path(__file__).parent.joinpath("samples").joinpath("sample-dotenv").joinpath(".env"),
    ):
        return super(FlaskConfig, cls).__new__(
            cls, yaml_path=yaml_path, dotenv_path=dotenv_path
        )

    @classmethod
    def select_config(cls, conf):
        conf = super(FlaskConfig, cls).select_config(conf)
        env = os.getenv("ENVIRONMENT")
        if env is not None:
            try:
                conf = conf[env]
                return conf
            except KeyError:
                raise NotImplementedError("The value set for 'ENVIRONMENT' attribute "
                                          "must match a section in your yaml config")
        else:
            raise NotImplementedError("You must set a variable called 'ENVIRONMENT', "
                                      "in your .env or by using "
                                      "'export ENVIRONMENT={yourEnvValue}, and "
                                      "{yourEnvValue} must match a section in your yaml "
                                      "file. If this doesn't make sense for your "
                                      "project, use SimpleConfig instead")

    @classmethod
    def set_dotenv_var_from_yaml_conf(cls, instance, conf, list_name="LOAD_FROM_ENV"):
        return super(FlaskConfig, cls).set_dotenv_var_from_yaml_conf(
            instance,
            conf,
            list_name=list_name
        )

    @property
    def SQLALCHEMY_DATABASE_URI(self):
        """
        Currently only supports SQLite and PostgreSQL.
        :return:
        """
        if self._db_uri is None:
            if self.DB_PREFIX == "sqlite:///":
                self._db_uri = f"{self.DB_PREFIX}" \
                               f"{os.path.join(self.SQLITE_PROJECT_DIRECTORY, self.DB_NAME)}"

            elif self.DB_PREFIX == "postgresql://":
                self._db_uri = f"{self.DB_PREFIX}{self.DB_USERNAME}:{self.DB_PASSWORD}@" \
                       f"{self.DB_SERVER}:{self.DB_PORT}/{self.DB_NAME}"

        return self._db_uri



import yaml
import os
from abc import ABC, abstractmethod
from dotenv import load_dotenv


class BaseConfig(ABC):
    _instance = None

    def __new__(cls, yaml_path, dotenv_path=None):
        if cls._instance is None:

            if dotenv_path is None:
                load_dotenv()
            else:
                load_dotenv(dotenv_path)

            with open(yaml_path) as f:
                conf = cls.select_config(yaml.safe_load(f))

            instance = super(BaseConfig, cls).__new__(cls)
            for k, v in conf.items():
                instance.__setattr__(k, v)

            instance = cls.set_dotenv_var_from_yaml_conf(instance, conf)

            cls._instance = instance

        return cls._instance

    @classmethod
    @abstractmethod
    def select_config(cls, conf):
        """
        Add custom logic here to return only a subset of the config, e.g., "development"
        :param conf: dict of config k/v
        :return: The sub-dict from which you want to directly set attributes for the
            instance
        """
        return conf

    @classmethod
    @abstractmethod
    def set_dotenv_var_from_yaml_conf(cls, instance, conf, list_name=None):
        """Override w/default list name if desired. Non-op as-is"""
        if list_name is not None:
            if list_name in conf:
                for attribute in conf[list_name]:
                    instance.__setattr__(attribute, os.getenv(attribute))
        return instance

    @classmethod
    def destroy(cls):
        cls._instance = None


import setuptools

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="yaml-pyconf",
    version="0.0.2",
    author="Gabrielle Sloane Law",
    author_email="rockymcrockerson@gmail.com",
    description="Turn YAML files into python objects",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/gabriellesw/yaml-pyconf",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Topic :: Utilities",
        "Development Status :: 3 - Alpha",
    ],
    python_requires=">=3.6",
    install_requires=["python-dotenv", "PyYAML"],
    test_suite="tests",
    package_data={
        "yaml_pyconf": [
            "samples/sample-yaml/*.yaml",
            "samples/sample-dotenv/.env",
        ]
    },
)



import pathlib

from yaml_pyconf.base import BaseConfig


class SimpleConfig(BaseConfig):

    def __new__(
            cls,
            yaml_path=pathlib.Path(__file__).parent.joinpath("samples").joinpath("sample-yaml").joinpath("simple.yaml")
    ):
        return super(SimpleConfig, cls).__new__(cls, yaml_path)

    @classmethod
    def select_config(cls, conf):
        return super(SimpleConfig, cls).select_config(conf)

    @classmethod
    def set_dotenv_var_from_yaml_conf(cls, instance, conf, list_name=None):
        return super(SimpleConfig, cls).set_dotenv_var_from_yaml_conf(instance, conf)
